"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const client = require("./client");
class BaseAction {
    constructor() {
        this.beforeChangeHandlers = [];
        this.afterChangeHandlers = [];
    }
    onChanged(handler) {
        this.afterChangeHandlers.push(handler);
    }
    onBeforeChange(handler) {
        this.beforeChangeHandlers.push(handler);
    }
    fireOnBeforeChange(issue) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const before of this.beforeChangeHandlers) {
                yield before(issue);
            }
        });
    }
    fireOnChanged(issue) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const after of this.afterChangeHandlers) {
                yield after(issue);
            }
        });
    }
}
exports.BaseAction = BaseAction;
class BaseIssueAction extends BaseAction {
    constructor(issue) {
        super();
        this.issue = issue;
    }
    fireOnBeforeChange() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("fireOnBeforeChange").call(this, this.issue);
        });
    }
    fireOnChanged() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("fireOnChanged").call(this, this.issue);
        });
    }
}
exports.BaseIssueAction = BaseIssueAction;
var Labels;
(function (Labels) {
    class Base extends BaseIssueAction {
        constructor(issue, labels) {
            super(issue);
            this.labels = labels;
        }
    }
    Labels.Base = Base;
    class Add extends Base {
        get summary() {
            return `Add labels ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                const labelsToAdd = this.labels.filter(lab => !this.issue.hasLabel(lab));
                if (labelsToAdd.length === 0)
                    return;
                yield this.fireOnBeforeChange();
                yield client.addLabels(this.issue, labelsToAdd);
                yield this.fireOnChanged();
            });
        }
    }
    Labels.Add = Add;
    class Remove extends Base {
        get summary() {
            return `Remove labels ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                const labelsToRemove = this.labels.filter(lab => this.issue.hasLabel(lab));
                if (labelsToRemove.length === 0)
                    return;
                yield this.fireOnBeforeChange();
                yield client.removeLabels(this.issue, labelsToRemove);
                yield this.fireOnChanged();
            });
        }
    }
    Labels.Remove = Remove;
    class Set extends Base {
        get summary() {
            return `Apply label set ${JSON.stringify(this.labels)} to issue ${this.issue.number}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                const desired = this.labels.slice().sort();
                const actual = this.issue.labels.map(l => l.name).sort();
                if (JSON.stringify(desired) === JSON.stringify(actual)) {
                    return;
                }
                yield this.fireOnBeforeChange();
                yield client.setLabels(this.issue, this.labels);
                yield this.fireOnChanged();
            });
        }
    }
    Labels.Set = Set;
})(Labels = exports.Labels || (exports.Labels = {}));
var Comments;
(function (Comments) {
    function parseHeader(body) {
        const regex = /^<!--header (.*) headerend-->/g;
        const match = regex.exec(body);
        return match ? JSON.parse(match[1]) : undefined;
    }
    function makeHeader(header) {
        // return `<!-- 🤖🔊 ${JSON.stringify(header)} 🔊🤖 -->`;
        return `<!--header ${JSON.stringify(header)} headerend-->`;
    }
    function makeFooter() {
        // return `<!-- 🤖🔈 potent-bot @ ${(new Date()).toLocaleString()} 🔈🤖 -->`;
        return `<!--footer potent-bot @ ${(new Date()).toLocaleString()} footerend-->`;
    }
    function makeComment(slug, body) {
        return makeHeader({ slug }) + '\r\n' + body + '\r\n' + makeFooter();
    }
    function getBody(comment) {
        //const regex = /🔊🤖 -->\r?\n(.*)\r?\n<!-- 🤖🔈/;
        const regex = /headerend-->\r?\n(.*)\r?\n<!--footer/;
        const match = regex.exec(comment.body);
        if (match) {
            return match[1];
        }
        return comment.body;
    }
    class Base extends BaseIssueAction {
        constructor(issue, slug) {
            super(issue);
            this.slug = slug;
        }
    }
    Comments.Base = Base;
    class Add extends Base {
        constructor(issue, slug, body) {
            super(issue, slug);
            this.body = body;
        }
        get summary() {
            return `Write comment (slug '${this.slug}') on issue ${this.issue.number}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                const me = yield client.getMyLogin();
                // Find my comments, if it exists
                const comments = (yield this.issue.getComments()).filter(c => c.user.login === me.login);
                for (const c of comments) {
                    const header = parseHeader(c.body);
                    if (header && (header.slug === this.slug)) {
                        const body = getBody(c);
                        if (body !== this.body) {
                            console.log('Actual: ' + body);
                            console.log('Desired: ' + this.body);
                            this.fireOnBeforeChange();
                            yield client.editComment(c, makeComment(this.slug, this.body));
                            this.fireOnChanged();
                        }
                        return;
                    }
                }
                this.fireOnBeforeChange();
                yield client.addComment(this.issue, makeComment(this.slug, this.body));
                this.fireOnChanged();
            });
        }
    }
    Comments.Add = Add;
})(Comments = exports.Comments || (exports.Comments = {}));
var Assignees;
(function (Assignees) {
    class Base extends BaseIssueAction {
        constructor(issue, assignees) {
            super(issue);
            this.assignees = assignees;
        }
    }
    Assignees.Base = Base;
    class Add extends Base {
        get summary() {
            return `Assign issue ${this.issue.fullName} to ${JSON.stringify(this.assignees)}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                throw new Error('Not implemented');
            });
        }
    }
    Assignees.Add = Add;
})(Assignees = exports.Assignees || (exports.Assignees = {}));
var Issues;
(function (Issues) {
    class Lock extends BaseIssueAction {
        get summary() {
            return `Lock issue ${this.issue.fullName}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.issue.locked)
                    return;
                this.fireOnBeforeChange();
                yield client.lockIssue(this.issue);
                this.fireOnChanged();
            });
        }
    }
    Issues.Lock = Lock;
    class Unlock extends BaseIssueAction {
        get summary() {
            return `Unlock issue ${this.issue.fullName}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.issue.locked)
                    return;
                this.fireOnBeforeChange();
                yield client.unlockIssue(this.issue);
                this.fireOnChanged();
            });
        }
    }
    Issues.Unlock = Unlock;
    class Close extends BaseIssueAction {
        get summary() {
            return `Close issue ${this.issue.fullName}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.issue.state === 'open') {
                    this.fireOnBeforeChange();
                    yield client.closeIssue(this.issue);
                    this.fireOnChanged();
                }
            });
        }
    }
    Issues.Close = Close;
    class Reopen extends BaseIssueAction {
        get summary() {
            return `Reopen issue ${this.issue.fullName}`;
        }
        execute(info) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.issue.state === 'closed') {
                    this.fireOnBeforeChange();
                    yield client.reopenIssue(this.issue);
                    this.fireOnChanged();
                }
            });
        }
    }
    Issues.Reopen = Reopen;
})(Issues = exports.Issues || (exports.Issues = {}));
