"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const client = require("./client");
const weak_string_map_1 = require("./weak-string-map");
const actionRunner_1 = require("./actionRunner");
const moment = require("moment");
const Actions = require("./action");
const Pools = require("./pools");
function parseDate(s) {
    if (s === null)
        return null;
    return moment(new Date(s));
}
function parseRepoReference(url) {
    // https://api.github.com/repos/octocat/Hello-World/something
    const regex = /^https:\/\/[^\/]+\/repos\/([^\/]+)\/([^\/]+)/;
    const match = regex.exec(url);
    if (match == null) {
        throw new Error(`Repository URL was in an unexpected format: ${url}`);
    }
    return Repository.create(match[1], match[2]);
}
exports.parseRepoReference = parseRepoReference;
class User {
    constructor(originalData) {
        this.originalData = originalData;
        this.update(originalData);
    }
    update(data) {
        Object.assign(this, {
            login: data.login
        });
    }
}
exports.User = User;
class Label {
    constructor(repo, data) {
        this.repo = repo;
        this.update(data);
    }
    update(data) {
        Object.assign(this, { name: data.name, color: data.color });
    }
}
exports.Label = Label;
class IssueComment {
    constructor(originalData, issue) {
        this.originalData = originalData;
        this.issue = issue;
        this.update(originalData);
    }
    update(data) {
        Object.assign(this, {
            id: data.id,
            user: Pools.Users.instantiate(data.user),
            created_at: parseDate(data.created_at),
            updated_at: parseDate(data.updated_at),
            body: data.body,
            repository: parseRepoReference(data.issue_url)
        });
    }
}
exports.IssueComment = IssueComment;
class Milestone {
    constructor(originalData) {
        this.originalData = originalData;
        Object.assign(this, {
            id: originalData.id,
            number: originalData.number,
            title: originalData.title,
            description: originalData.description,
            state: originalData.state
        });
        this.creator = Pools.Users.instantiate(originalData.creator);
        this.created_at = parseDate(originalData.created_at);
        this.updated_at = parseDate(originalData.updated_at);
        this.closed_at = parseDate(originalData.closed_at);
        this.due_on = parseDate(originalData.closed_at);
    }
}
exports.Milestone = Milestone;
class Repository {
    constructor(owner, name) {
        this.owner = owner;
        this.name = name;
    }
    static create(owner, name) {
        const key = owner + '/' + name;
        const result = Repository.cache.get(key);
        if (result === undefined) {
            const repo = new Repository(owner, name);
            Repository.cache.set(key, repo);
            return repo;
        }
        return result;
    }
}
Repository.cache = new weak_string_map_1.default();
exports.Repository = Repository;
class Issue {
    constructor(originalData) {
        this.originalData = originalData;
        // Copy some fields
        Object.assign(this, {
            number: originalData.number,
            title: originalData.title,
            body: originalData.body,
            state: originalData.state,
            locked: originalData.locked
        });
        this.repository = parseRepoReference(originalData.repository_url);
        // Parse some dates
        this.created_at = parseDate(originalData.created_at);
        this.updated_at = parseDate(originalData.updated_at);
        this.closed_at = parseDate(originalData.closed_at);
        // Intern some instances
        this.user = Pools.Users.instantiate(originalData.user);
        this.labels = originalData.labels.map(l => Pools.Labels.get(this.repository, l));
        this.assignees = originalData.assignees.map(user => Pools.Users.instantiate(user));
        // Set the PR flag
        this.isPullRequest = !!originalData.pull_request;
    }
    /** Returns a string like 'Microsoft/TypeScript#14' */
    get fullName() {
        return `${this.repository.owner}/${this.repository.name}#${this.number}`;
    }
    update() {
        // TODO: impl
    }
    getComments() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield client.fetchIssueComments(this);
            return data.map(raw => Pools.IssueComments.instantiate(raw, this));
        });
    }
    /**
    * Adds a label to an issue or PR.
    */
    addLabel(...labels) {
        return actionRunner_1.addAction(new Actions.Labels.Add(this, labels));
    }
    /**
     * Adds labels to an issue or PR.
     */
    addLabels(labels) {
        return actionRunner_1.addAction(new Actions.Labels.Add(this, labels));
    }
    /**
     * Deletes a label from an issue.
     */
    removeLabel(...labels) {
        return actionRunner_1.addAction(new Actions.Labels.Remove(this, labels));
    }
    /**
     * Deletes labels from an issue.
     */
    removeLabels(labels) {
        return actionRunner_1.addAction(new Actions.Labels.Remove(this, labels));
    }
    /**
     * Sets, exactly, which labels are on an issue
     */
    setLabels(...labels) {
        return actionRunner_1.addAction(new Actions.Labels.Set(this, labels));
    }
    /**
     * Returns true if this issue has the specified label
     */
    hasLabel(labelName) {
        const name = typeof labelName === 'string' ? labelName : labelName.name;
        return this.labels.some(l => l.name === name);
    }
    /**
     * Adds or updates a comment with the specified slug to the issue
     */
    addComment(slug, body) {
        return actionRunner_1.addAction(new Actions.Comments.Add(this, slug, body));
    }
    lock() {
        return actionRunner_1.addAction(new Actions.Issues.Lock(this));
    }
    unlock() {
        return actionRunner_1.addAction(new Actions.Issues.Unlock(this));
    }
    close() {
        return actionRunner_1.addAction(new Actions.Issues.Close(this));
    }
    reopen() {
        return actionRunner_1.addAction(new Actions.Issues.Reopen(this));
    }
}
exports.Issue = Issue;
